/**************************************************************************************************/
/***                                 history                                                    ***/
/**************************************************************************************************/
/*** 04.08.1996: command added: rep string id [FF|LF|EF] with string a                          ***/
/***             options for add comand added:                                                  ***/
/***             - NOXP: nop if exist at position: equal to NOX, only for compatibility         ***/
/***             - NOXS: nop if exist in section                                                ***/
/***                 add line in section: NOX, NOXP, NOXS have same meening                     ***/
/***             - NOXF: nop if exist in file                                                   ***/
/***                 add line/section after/before line:                                        ***/
/***                     NOXF looks up the whole file, without looking for any sequence of      ***/
/***                     lines, while NOX looks for position and sequence                       ***/
/***                 add section after/before section:                                          ***/
/***                     NOX, NOXP, NOXF have same meening.                                     ***/
/***                     only the section name is checked in the whole file                     ***/
/***             revision changed to 1.1                                                        ***/
/**************************************************************************************************/
Call RxFuncAdd 'SysLoadFuncs','REXXUTIL','SysLoadFuncs'
Call SysLoadFuncs;
call Logo

/*------------------------------------------------------------------------------------------------*/
esc          = x2c('1B')
Color.cyan   = esc'[1;36;40m'
Color.white  = esc'[0;37;40m'
Color.red    = esc'[1;31;40m'
Color.yellow = esc'[1;33;40m'
Color.green  = esc'[1;32;40m'
Color.grey   = esc'[1;30;40m'
/*------------------------------------------------------------------------------------------------*/
Command.0 = 7                                                                   /* valid commands */
Command.1 = 'OPEN  O'
Command.2 = 'CLOSE C'
Command.3 = 'ADD   A'
Command.4 = 'DEL   DELETE D'
Command.5 = 'REP   REPLACE R MODIFY MOD M'
Command.6 = 'SAV   SAVE S'
Command.7 = 'REX   REXX'

/*------------------------------------------------------------------------------------------------*/
Option.0  = 16                                                                   /* valid options */
Option.1  = 'ATB  ATBEGIN'
Option.2  = 'ATB  ATBEGIN'
Option.3  = 'ATE  ATEND'
Option.4  = 'AFT  AFTER'
Option.5  = 'BEF  BEFORE'
Option.6  = 'NOX  NOXP NOPIFEXIST NOPIFEXISTATPOSITION'
Option.7  = 'NOXS NOPIFEXISTINSECTION'
Option.8  = 'NOXF NOPIFEXISTINFILE'
Option.9  = 'DOX  DOIFEXIST DOANYWAY'
Option.10 = 'FF   FIRSTFOUND'
Option.11 = 'LF   LASTFOUND'
Option.12 = 'EF   EVERYFOUND'
Option.13 = 'ICR  IFNOTFOUNDCREATE'
Option.14 = 'WIT  WITH'
Option.15 = 'IN'
Option.16 = 'NOF  NOPIFNOTFOUND'

/*------------------------------------------------------------------------------------------------*/
Attribs = 'IN AFTER BEFORE WITH AFT BEF WIT'                                 /* valid  attributes */
Types   = 'STRING LINE SECTION TOKEN'                                              /* valid types */
/*------------------------------------------------------------------------------------------------*/
/* command structure                                                                              */
C.Cmd     = ''                                                                         /* Command */
C.Type1   = ''                                                                    /* Type of What */
C.What.0  = 0                                                      /* several what lines possible */
C.Option1 = ''                                                                   /* first options */
C.Attr    = ''                                                                   /* where or with */
C.Type2   = ''                                                                  /* Type of target */
C.Search  = ''                                                               /* search identifier */
C.Option2 = ''                                                                  /* search options */
/*------------------------------------------------------------------------------------------------*/
SAVE.0    = 0                                                         /* no save string specified */
/*------------------------------------------------------------------------------------------------*/

do queued(); pull; end                                                        /* clear rexx queue */

params = arg(1)
CFile = HandleParametersAndInputErrors(params)/* return input file if exist, else command ar queud */
if CFile <> '' then do                                                /* queue input command file */
    do while lines(CFile)
        ll = linein(CFile)
        queue ll
    end
    call stream CFile, 'c', 'close'                       /* and close it, cause we are kind guys */
end

AlreadyRead = 0
do while queued() > 0
    if AlreadyRead == 0 then parse pull OrgLine
    AlreadyRead = 0
    OrgLine = RemoveComment( OrgLine )
    if OrgLine == '' then iterate
    UppLine = translate( OrgLine )
    Command = ValidCommand(word( Uppline, 1 ))
    if Command <> '' then do
        CommandLine = OrgLine
        do while lines( CFile ) & substr(CommandLine, length(CommandLine)) == ','
            parse pull OrgLine
            OrgLine = RemoveComment( OrgLine )
            if OrgLine == '' then iterate
            UppLine = translate( OrgLine )
            if OrgLine <> '' then do
                CommandLine = CommandLine || OrgLine
            end
        end
/*      say
        say 'Original:' CommandLine */
        C.Cmd     = ''
        C.Type1   = ''
        C.What.0  = 0
        C.Option1 = ''
        C.Attr    = ''
        C.Type2   = ''
        C.Search  = ''
        C.Option2 = ''
        select
            /*------------------------------------------------------------------------------------*/
            when Command == 'OPEN' then do                                                /* OPEN */
                C.Cmd = Command
                if ParseFileCommand( CommandLine ) == 0 then do
                    ret = ValidateCommand()
                    if ret <> 0 then /* say ret ':' CommandLine */ NOP
                                else ret = OpenClose('OPEN')
                end
            end
            /*------------------------------------------------------------------------------------*/
            when Command == 'CLOSE' then do                                              /* CLOSE */
                C.Cmd = Command
                if ParseFileCommand( CommandLine ) == 0 then do
                    ret = ValidateCommand()
                    if ret <> 0 then /* say ret ':' CommandLine */ NOP
                                else ret = OpenClose('CLOSE')
                end
            end
            /*------------------------------------------------------------------------------------*/
            when Command == 'ADD' then do                                                  /* ADD */
                C.Cmd = Command
                if ParseADDCommand( CommandLine ) == 0 then do
                    ret = ValidateCommand()
                    if ret <> 0 then /* say ret ':' CommandLine */ NOP
                                else ret = Add()
                end
            end
            /*------------------------------------------------------------------------------------*/
            when Command == 'DEL' then do                                                  /* DEL */
                C.Cmd = Command
                if ParseDRSCommand( CommandLine ) == 0 then do
                    ret = ValidateCommand()
                    if ret <> 0 then say ret ':' CommandLine
                                else ret = Delete()
                end
            end
            /*------------------------------------------------------------------------------------*/
            when Command == 'REP' then do                                                  /* REP */
                C.Cmd = Command
                if ParseDRSCommand( CommandLine ) == 0 then do
                    ret = ValidateCommand()
                    if ret <> 0 then say ret ':' CommandLine
                                else ret = Replace()
                end
            end
            /*------------------------------------------------------------------------------------*/
            when Command == 'SAV' then do                                                  /* SAV */
                C.Cmd = Command
                if ParseDRSCommand( CommandLine ) == 0 then do
                    ret = ValidateCommand()
                    if ret <> 0 then say ret ':' CommandLine
                                else ret = Save()
                end
            end
            /*------------------------------------------------------------------------------------*/
            when Command == 'REX' then do                                                  /* REX */
                C.Cmd = Command
                if ParseREXXCommand( CommandLine ) == 0 then do
                    ret = ValidateCommand()
                    if ret <> 0 then say ret ':' CommandLine
                                else ret = Rexx()
                end
            end
            /*------------------------------------------------------------------------------------*/
            otherwise NOP
        end
    end
    else do
        say Color.red||'invalid command:' word(UppLine,1)||Color.white
    end
end

exit 0

/**************************************************************************************************/
AnalyseWhatString: procedure expose C. SAVE. Color.
cc = arg(1)
cc = strip( cc )
i = 1
C.What.1 = ''
do while substr(cc,1,1) == '"' | substr(cc,1,1) == ',' | translate(substr(cc,1,4)) == 'SAVE'
    if substr( cc,1,1 ) == '"' then do
        parse value cc with '"' token '"' cc
        C.What.i = C.What.i || token
        do while substr( cc,1,1 ) == '"'                            /* must be a '""' what is '"' */
            parse value cc with '"' token '"' cc
            C.What.i = C.What.i || '"' ||token
        end
    end
    else if substr( cc,1,1 ) == ',' then do
        i = i+1
        C.What.i = ''
        cc = substr( cc,2 )
    end
    else if substr( cc,1,4 ) == 'SAVE' then do
        if SAVE.0 > 0 then do
            if SAVE.1 <> '' then C.What.i = C.What.i || SAVE.1
            do k = 2 to SAVE.0
                i = i+1
                C.What.i = SAVE.k
            end
            cc = substr( cc,5 )
        end
    end
    cc = strip( cc )
end
C.What.0 = i
return cc
/**************************************************************************************************/
ParseADDCommand: procedure expose C. Option. SAVE. Color. Color.
cc = arg(1)
tmpc = cc

parse value cc with gagacmd cc                                                 /* extract command */
parse value cc with C.Type1 cc
C.Type1 = translate(C.Type1)

cc = AnalyseWhatString( cc )

cc = strip( cc )
if substr( cc,1,1 ) == '(' then parse value cc with '(' C.Option1 ')' cc
cc = strip( cc )
parse value cc with C.Attr C.Type2 cc
C.Attr  = translate(C.Attr)
C.Type2 = translate(C.Type2)
cc = strip( cc )
if lastpos(')',cc) == length(cc) then do                             /* there is a leading option */
    parse value substr(cc,lastpos('(',cc)) with '(' C.Option2 ')'
    C.Search = strip(substr(cc,1,lastpos('(',cc)-1))
end
else C.Search = strip(cc)

if ValidateOption() == -1 then do
    say Color.yellow||'unknown option or attribute in'||Color.white
    say tmpc
end
return 0

/**************************************************************************************************/
ParseDRSCommand: procedure expose C. Attribs Option. SAVE. Color.
cc = arg(1)
tmpc = cc

parse value cc with gagacmd cc                                                 /* extract command */
parse value cc with C.Type1 cc
C.Type1 = translate(C.Type1)

cc = strip( cc )

tmpcc = cc
do while pos('"',tmpcc) >0
    parse value tmpcc with tt1'"'id'"'tt2
    tmpcc = tt1||copies('x',length(id)+2)||tt2
end

do i = 1 to words( Attribs )                                /* find position of 2. part of string */
    pp = wordpos(word(Attribs,i),translate(tmpcc))
    if pp >0 then do
        pp = wordindex(tmpcc,pp)
        part1 = strip(substr(cc,1,pp-1))
        part2 = strip(substr(cc,pp))
        leave
    end
end
if pp == 0 then do
    part1 = strip(cc)
    part2 = ''
end

if length(part1) > 0 then do
    if lastpos(')',part1) == length(part1) then do                   /* there is a leading option */
        parse value substr(part1,lastpos('(',part1)) with '(' C.Option1 ')'
        C.What.1 = strip(substr(part1,1,lastpos('(',part1)-1))
    end
    else C.What.1 = strip(part1)
    C.What.0 = 1
end

if length(part2) > 0 then do
    parse value part2 with C.Attr C.Type2 part2
    C.Attr  = translate(C.Attr)
    C.Type2 = translate(C.Type2)
    if lastpos(')',part2) == length(part2) then do                   /* there is a leading option */
        parse value substr(part2,lastpos('(',part2)) with '(' C.Option2 ')'
        C.Search = strip(substr(part2,1,lastpos('(',part2)-1))
    end
    else C.Search = strip(part2)
end

if C.Cmd == 'REP' then do
    tmp = C.Search                                                     /* this is the repl string */
    C.Search = C.What.1
    if tmp <> '' then do
        tmp = AnalyseWhatString( tmp )                                 /* writes the C.What. stem */
    end
end

if ValidateOption() == -1 then say Color.yellow||'unknown option in'||Color.white tmpc
return 0

/**************************************************************************************************/
ParseFileCommand: procedure expose C.
cc = arg(1)
parse value cc with gagacmd cc                                                 /* extract command */
parse value cc with C.What.1 cc
C.What.0 = 1
return 0
/**************************************************************************************************/
ParseREXXCommand: procedure expose C. SAVE.
cc = arg(1)
parse value cc with gagacmd cc                                                 /* extract command */
cc = AnalyseWhatString( cc )
return 0
/**************************************************************************************************/
Rexx: procedure expose C. Color.
RexxCommandLine_ = ''
do _i_ = 1 to C.What.0                              /* special count variable to have no conflict */
    RexxCommandLine_ = RexxCommandLine_ || ';' || C.What._i_
end
signal on SYNTAX  NAME interpretsyntaxerror
signal on ERROR   NAME interpretsyntaxerror
signal on FAILURE NAME interpretsyntaxerror
interpret RexxCommandLine_
signal off SYNTAX
return 0
/**************************************************************************************************/
interpretsyntaxerror:
say Color.red||'---> ERROR in REXX command to interprete'||Color.white
return -1
/**************************************************************************************************/
RemoveComment: procedure
line = arg(1)
AnyComment = pos('//',line)
if AnyComment <> 0 then line = substr(line,1,AnyComment-1)
if pos( '/*', line ) <> 0 then do
    line = strip(substr(line,1,pos('/*',line)-1)) ||,
           strip(substr(line,pos('*/',line)+2))
end
line = strip( line )
return line
/**************************************************************************************************/
ValidCommand: procedure expose Command. Color.
cc = arg(1)
do i = 1 to Command.0
    if wordpos( cc, Command.i ) <> 0 then return word(Command.i,1)
end
return ''
/**************************************************************************************************/
OpenClose: procedure expose C. Line. CloseFileName Color.
command = strip(arg(1))
FileName = C.What.1
if C.Cmd == 'OPEN' then do
    if stream(FileName,'c','query exists') <> '' then do
        i=0
        do while lines( FileName )
          i=i+1
          Line.i = linein( FileName )
        end
        Line.0 = i
        call stream FileName, 'C', 'CLOSE'
        CloseFileName = FileName
        /* say '---> file read:' FileName */
    end
    else do
        Line.0 = 0
        CloseFileName = FileName
        /* say '---> file does not exist, will be created when closing:' FileName */
        return -1
    end
end
else do
    if FileName <> '' then CloseFileName = FileName
    '@del' CloseFileName '1>NUL 2>NUL'
    do i = 1 to Line.0
        call lineout CloseFileName, Line.i
    end
    if stream(CloseFileName,'c','close') == 'READY:' then do
        /* say '---> file successfully written:' CloseFileName */
    end
    /* else say Color.yellow||'---> file not written:'||Color.white CloseFileName */
end

return 0
/**************************************************************************************************/
Replace: procedure expose Line. LinesFound. C. Color.
Insert.0 = C.What.0
do i = 1 to Insert.0
    Insert.i = C.What.i
end
select
   when C.Type1 == 'STRING' then do
       LS = C.Search || '*'
       LS = insert( '*', LS, pos('"', LS)-1)                         /* add *'s to find the lines */
       m = LookForLines( LS, C.Option1 )          /* look for lines, already with the same option */
       if C.Option1 == 'FF' then do                                  /* FF string is FF line also */
       snum = LookForStrings('ID', C.Search, COption1, LinesFound.1 )  /* look for string in line */
           if snum == 1 then do                                         /* only one must be found */
               k = LinesFound.1                                                 /* number of line */
               Line.k = substr(Line.k,1,StringsFound.1.Start-1)||C.What.1||substr(Line.k,StringsFound.1.End+1)
               /* say '---> string replaced in line nr.'k 'position' StringsFound.1.Start */
           end
       end
       if C.Option1 == 'LF' then do                                              /* same as above */
       snum = LookForStrings('ID', C.Search, COption1, LinesFound.1 )  /* look for string in line */
           if snum == 1 then do
               k = LinesFound.m
               Line.k = substr(Line.k,1,StringsFound.snum.Start-1)||C.What.1||substr(Line.k,StringsFound.snum.End+1)
               /* say '---> string replaced in line nr.'k 'position' StringsFound.snum.Start */
           end
       end
       if C.Option1 == 'EF' then do
           do i = m to 1 by -1
               k = LinesFound.i
               snum = LookForStrings('ID', C.Search, COption1, LinesFound.i )/* look for string in line */
               do l = snum to 1 by -1
                   Line.k = substr(Line.k,1,StringsFound.l.Start-1)||C.What.1||substr(Line.k,StringsFound.l.End+1)
                   /* say '---> string replaced in line nr.'k 'position' StringsFound.l.Start */
               end
           end
       end
   end
   when C.Type1 == 'LINE' then do
       m = LookForLines( C.Search, C.Option1 )
       do i = m to 1 by -1
           call InsertLine LinesFound.i                                     /* insert after found */
           call DeleteLine LinesFound.i, 1                                        /* delete found */
           /* say '---> replace line nr.'LinesFound.i */
       end
       if m == 0 then do                                             /* if no matching line found */
           say Color.yellow||'---> target line not found:'||Color.white C.Search
           if pos('ICR ATB',C.Option1) > 0 then do                            /* create at bottom */
               call InsertLine 0                                               /* insert at begin */
               /* say '---> insert line at begin' */
           end
           if pos('ICR ATE',C.Option1) > 0 then do                               /* create at top */
               call InsertLine Line.0                                            /* insert at end */
               /* say '---> insert line at end' */
           end
       end
   end
   when C.Type1 == 'SECTION' then do
       snum = LookForSections( C.Search, C.Option1, 1 )
       do i = snum to 1 by -1
           call DeleteLine SectionsFound.i.Start, SectionsFound.i.End - SectionsFound.i.Start +1
           call InsertLine SectionsFound.i.Start -1
           /* say '---> replace section from line nr.'SectionsFound.i.Start 'to line nr.'SectionsFound.i.End */
       end
       if snum == 0 then do                                          /* if no matching line found */
           say Color.yellow||'---> target section not found:'Color.white C.Search
           if pos('ICR ATB',C.Option1) > 0 then do                            /* create at bottom */
               call InsertLine 0                                               /* insert at begin */
               /* say '---> insert section at begin' */
           end
           if pos('ICR ATE',C.Option1) > 0 then do                               /* create at top */
               call InsertLine Line.0                                            /* insert at end */
               /* say '---> insert section at end' */
           end
       end
   end
   otherwise NOP
end
return 0
/**************************************************************************************************/
Add: procedure expose Line. LinesFound. C. Color.
Insert.0 = C.What.0                                             /* prepare insertion stem Insert. */
do i = 1 to Insert.0
    Insert.i = C.What.i
end
select
    when C.Type1 == 'STRING' | C.Type1 == 'TOKEN' then do
        lnum = LookForLines( C.Search, C.Option2 )
        if lnum == 0 then do
            say Color.yellow||'---> target line not found:'||Color.white C.Search '-->NOP'
        end
        do i = lnum to 1 by -1                                            /* for all target lines */
            if C.Type1 == 'TOKEN' then do             /* for tokens, build complete search string */
                cstring = ''
                do o = 1 to C.What.0
                    cstring = cstring || C.What.o
                end
            end
            else cstring = C.What.1                  /* for strings, only one string is specified */
            snum = LookForStrings('AS', cstring, 'FF', LinesFound.i )           /* already exists */
            if snum > 0 & pos('NOX',C.Option1) >0 then do
                say '---> string/token already exists in line nr.'LinesFound.i '-->NOP'
                iterate i                                       /* if str exist and NOX specified */
            end
            if pos('ATB',C.Option1) >0 | pos('ATE',C.Option1) >0 then do     /* if ATB,ATF, !=NOX */
                k = LinesFound.i                                                   /* line number */
                if pos('ATB',C.Option1) >0 then do
                    Line.k = cstring || Line.k                                 /* insert at begin */
                    /* say '---> string/token inserted in line nr.'k */
                end
                if pos('ATE',C.Option1) >0 then do
                    if C.Type1 == 'TOKEN' then do
                        if C.What.0 > 1 & lastpos(C.What.2,Line.k) <> length(Line.k)
                        then Line.k = Line.k || C.What.2 || cstring              /* insert at end */
                        else Line.k = Line.k || cstring /* insert at end without token terminator */
                        /* say '---> token inserted in line nr.'k */
                    end
                    else do
                        Line.k = Line.k || cstring                               /* insert at end */
                        /* say '---> string inserted in line nr.'k */
                    end
                end
            end
            else do
                if pos('AFT',C.Option1) >0 | pos('BEF',C.Option1) >0 then do
                    startpos = pos('"', C.Option1)                            /* look for first " */
                    endpos   = lastpos('"',C.Option1)                          /* look for last " */
                    if  startpos <> 0 then do
                        sstring = substr(C.Option1,startpos,endpos-startpos+1)/* string between " " */
                        if translate(substr(C.Option1,startpos-1,1)) == 'C' then do/* look for leading c */
                            sstring = 'C' || sstring
                            startpos = startpos-1
                        end
                    end
                    else sstring = '""'                                          /* no " " string */
                    soption = ''
                    if pos(' FF',C.Option1) >0 then soption = 'FF'
                    if pos(' LF',C.Option1) >0 then soption = 'LF'
                    if pos(' EF',C.Option1) >0 then soption = 'EF'     /* EF,BEF look the same!!! */
                    if soption == '' then soption = 'FF'              /* set default value FF $$ */
                    tsum = LookForStrings('ID', sstring, soption, LinesFound.i )
                    if tsum == 0 then do
                        say Color.yellow||'---> string' sstring 'to insert after/before not found:'||Color.white
                    end
                    k = LinesFound.i                                               /* line number */
                    if C.Type1 == 'STRING' then toinsert = C.What.1
                                           else toinsert = C.What.1||C.What.2
                    do l = tsum to 1 by -1
                        if pos('AFT',C.Option1) >0 then do
                            Line.k = substr(Line.k,1,StringsFound.l.End)||toinsert||substr(Line.k,StringsFound.l.End+1)
                            /* say '---> string/token inserted in line nr.'k */
                        end
                        if pos('BEF',C.Option1) >0 then do
                            Line.k = substr(Line.k,1,StringsFound.l.Start-1)||toinsert||substr(Line.k,StringsFound.l.Start)
                            /* say '---> string/token inserted in line nr.'k */
                        end
                    end
                end
            end
        end
    end
    when C.Type1 == 'LINE' | C.Type1 == 'SECTION' then do
        select
            when (C.Attr == 'AFT' | C.Attr == 'BEF') & (C.Type2 == 'LINE') then do
                if pos('NOXF',C.Option1) > 0 then do                     /* if NOPIFEXIST in file */
                    identical = 0
                    if C.Type1 == 'LINE' then do
                        do j = 1 to C.What.0                  /* look up the whole file, unsorted */
                            do k = 1 to Line.0
                                if translate(C.What.j) == translate(Line.k) then identical = identical +1
                            end
                            if identical == C.What.0 then do
                                say Color.yellow||'---> line(s) to insert already exist in the file (unsorted, uppercase compare) --> NOP'||Color.white
                                return 0                                        /* all match->NOP */
                            end
                        end
                    end
                    else do                                                            /* section */
                        do k = 1 to Line.0                      /* look only for the section name */
                            if translate(C.What.1) == translate(Line.k) then do
                                say Color.yellow||'---> name of section to insert already exist in the file (uppercase compare) --> NOP'||Color.white
                                return 0                                        /* all match->NOP */
                            end
                        end
                    end
                end
                lnum = LookForLines( C.Search, C.Option2 )                     /* look for target */
                do i = lnum to 1 by -1                                    /* for all target lines */
                    if pos('NOX',C.Option1) > 0 then do                          /* if NOPIFEXIST */
                        identical = 0                                        /* identical counter */
                        do j = 1 to C.What.0                           /* for all lines to insert */
                            if C.Attr == 'AFT' then do                     /* compare lines after */
                                index = LinesFound.i +j                          /* line in Line. */
                                if translate(C.What.j) == translate(Line.index) then identical = identical +1
                            end
                            if C.Attr == 'BEF' then do                    /* compare lines before */
                                index = LinesFound.i - C.What.0 -1 +j              /* lines above */
                                if translate(C.What.j) == translate(Line.index) then identical = identical +1
                            end
                        end
                        if identical == C.What.0 then do
                            say Color.yellow||'---> line/section to insert already exist (uppercase compare) --> NOP'||Color.white
                            iterate i                                           /* all match->NOP */
                        end
                    end
                    if C.Attr == 'AFT' then do                         /* if not all match or DOX */
                        call InsertLine LinesFound.i                              /* insert after */
                        /* say '---> line/section inserted after line nr.'LinesFound.i */
                    end
                    if C.Attr == 'BEF' then do
                        call InsertLine LinesFound.i -1                          /* insert before */
                        /* say '---> line/section inserted before line nr.'LinesFound.i */
                    end
                end
                if lnum == 0 then do                                 /* if no matching line found */
                    say Color.yellow||'---> target line not found:'||Color.white C.Search
                    if pos('ICR ATB',C.Option1) > 0 then do                   /* create at bottom */
                        call InsertLine 0                                      /* insert at begin */
                        /* say '---> line/section inserted at begin of file' */
                    end
                    if pos('ICR ATE',C.Option1) > 0 then do                      /* create at top */
                        call InsertLine Line.0                                   /* insert at end */
                        /* say '---> line/section inserted at end of file' */
                    end
                end
            end
            when C.Type1 == 'LINE' & C.Attr == 'IN'  & C.Type2 == 'SECTION' then do
                snum = LookForSections( C.Search, C.Option2, 1 )
                if snum == 0 then do
                    say Color.yellow||'---> target section not found:'||Color.white C.Search
                end
                if snum == 0 & pos('NOF',C.Option2) == 0 then do /* section not found and not NOF */
                    if pos('ICR ATB',C.Option2) >0 then insertat = 0
                    if pos('ICR ATE',C.Option2) >0 then insertat = Line.0
                    Insert.0 = 1;                                        /* 1 line = section name */
                    parse value C.Search with '"'Insert.1'"'       /* parse only the section name */
                    call InsertLine insertat                 /* insert the not found section name */
                    /* say '---> create new section named:' Insert.1 */
                    Insert.0 = C.What.0                   /* again prepare insertion stem Insert. */
                    do i = 1 to Insert.0
                        Insert.i = C.What.i
                    end
                    snum = 1
                    SectionsFound.1.Start = insertat +1             /* simulate one section found */
                    SectionsFound.1.End   = insertat +1                   /* the one just created */
                end
                do i = snum to 1 by -1                        /* NOX, NOXS have same meening here */
                    if pos('NOX',C.Option1) > 0 | pos('NOXS',C.Option1) > 0 then do/* if NOPIFEXIST */
                        identical = 0                                        /* identical counter */
                        do j = 1 to C.What.0                           /* for all lines to insert */
                            do k = SectionsFound.i.Start to SectionsFound.i.End
                                if translate(C.What.j) == translate(Line.k) then identical = identical +1/* line exist */
                            end
                        end
                        if identical == C.What.0 then do
                            say '---> line(s) already exist in section (uppercase compare) --> NOP'
                            iterate i                                           /* all match->NOP */
                        end
                    end
                    if pos('ATB',C.Option1) >0 then insertat = SectionsFound.i.Start
                    if pos('ATE',C.Option1) >0 then insertat = SectionsFound.i.End
                    call InsertLine insertat
                    /* say '---> insert line(s) at line nr.'insertat */
                end
            end
            when C.Type1 == 'SECTION' & (C.Attr == 'AFT' |C.Attr == 'BEF')  & C.Type2 == 'SECTION' then do
                snum = LookForSections( '"'C.What.1'"', 'FF', 1 )        /* look if already exist */
                if snum > 0 then do
                    if pos('NOX',C.Option1) > 0 | pos('NOXF',C.Option1) > 0 then do/* if NOPIFEXIST in file */
                        say Color.yellow||'---> name of section to insert already exist in the file (uppercase compare) --> NOP'||Color.white
                    end
                    if pos('DOX',C.Option1) > 0 then do
                        say Color.yellow||'---> section already exist'||Color.white
                    end
                end
                if snum == 0 | pos('DOX',C.Option1) > 0 then do          /* insert section anyway */
                    snum = LookForSections( C.Search, C.Option2, 1 )
                    if snum == 0 & pos('NOF',C.Option2) == 0 then do/* section not found and not NOF */
                        if pos('ICR ATB',C.Option1) >0 then insertat = 0       /* insert at begin */
                        if pos('ICR ATE',C.Option1) >0 then insertat = Line.0    /* insert at end */
                        call InsertLine insertat                                     /* insert it */
                        /* say '---> insert section at line nr.'insertat */
                    end
                    do i = snum to 1 by -1                             /* target section(s) found */
                        if C.Attr == 'AFT' then do
                            call InsertLine SectionsFound.i.End
                            /* say '---> insert section at line nr.'SectionsFound.i.End */
                        end
                        if C.Attr == 'BEF' then do
                            call InsertLine SectionsFound.i.Start -1
                            /* say '---> insert section at line nr.'SectionsFound.i.Start -1 */
                        end
                    end
                end
            end
            otherwise NOP
        end
    end
    otherwise NOP
end
return 0
/**************************************************************************************************/
Delete: procedure expose Line. LinesFound. C. Color.
select
   when C.Type1 == 'LINE' then do
       select
          when C.Attr == '' then do
              m = LookForLines( C.What.1, C.Option1 )
              if m == 0 then say Color.yellow||'---> target line not found:'||Color.white C.What.1
              do i = m to 1 by -1
                  say '---> delete line nr.'LinesFound.i
                  call DeleteLine LinesFound.i, 1
              end
          end
          when C.Attr == 'BEF' & C.Type2 == 'LINE' then do
              m = LookForLines( C.Search, C.Option2 )
              if m == 0 then say Color.yellow||'---> target line not found:'||Color.white C.Search
              do i = m to 1 by -1
                  say '---> delete line nr.'LinesFound.i -1
                  call DeleteLine LinesFound.i -1, 1
              end
          end
          when C.Attr == 'AFT'  & C.Type2 == 'LINE' then do
              m = LookForLines( C.Search, C.Option2 )
              if m == 0 then say Color.yellow||'---> target line not found:'||Color.white C.Search
              do i = m to 1 by -1
                  say '---> delete line nr.'LinesFound.i +1
                  call DeleteLine LinesFound.i +1, 1
              end
          end
          when C.Attr == 'IN'     & C.Type2 == 'SECTION' then do
              snum = LookForSections( C.Search, C.Option2, 1 )
              if m == 0 then say Color.yellow||'---> target section not found:'||Color.white C.Search
              do i = snum to 1 by -1
                  lnum = LookForLines( C.What.1, C.Option1, SectionsFound.i.Start, SectionsFound.i.End )
                  if lnum == 0 then say Color.yellow||'---> target section not found:'||Color.white C.What.1
                  do j = lnum to 1 by -1
                      k = LinesFound.j
                      say '---> delete line nr.'LinesFound.j
                      call DeleteLine LinesFound.j, 1
                  end
              end

          end
          otherwise NOP
       end
   end
   when C.Type1 == 'STRING' then do
       select
          when C.Attr == 'IN' & C.Type2 == 'LINE' then do
              lnum = LookForLines( C.Search, C.Option2 )
              if lnum == 0 then say Color.yellow||'---> target line not found:'||Color.white C.Search
              do i = 1 to lnum                                        /* for all lines that match */
                  snum = LookForStrings('ID', C.What.1, C.Option1, LinesFound.i )
                  if snum == 0 then say Color.yellow||'---> line' LinesFound.i': target string not found:'||Color.white C.What.1
                  do j = StringsFound.0 to 1 by -1
                      k = LinesFound.i
                      Line.k = substr(Line.k,1,StringsFound.j.Start-1) ||,
                               substr(Line.k,StringsFound.j.End+1)
                      say '---> delete string in line nr.'k
                  end
              end
          end
          otherwise NOP
      end
   end
   when C.Type1 == 'SECTION' then do
       select
          when C.Attr == '' then do
              snum = LookForSections( C.What.1, C.Option1, 1 )
              if snum == 0 then say Color.yellow||'---> target section not found:'||Color.white C.What.1
              do i = snum to 1 by -1
                  call DeleteLine SectionsFound.i.Start, SectionsFound.i.End - SectionsFound.i.Start +1
                  say '---> delete section from line nr.'SectionsFound.i.Start 'to line nr.'SectionsFound.i.End
              end
          end
          when C.Type2 == 'SECTION' then do                                  /* AFT & BEF SECTION */
              snum = LookForSections( C.Search, C.Option2, 1 )                    /* find section */
              if snum == 0 then say Color.yellow||'---> target section not found:'||Color.white C.Search
              found.0 = snum                                             /* num of sections found */
              do i = snum to 1 by -1
                  found.i.start = SectionsFound.i.Start                    /* remember start line */
                  found.i.end   = SectionsFound.i.End                        /* remember end line */
              end
              if C.Attr == 'AFT' then do                                         /* AFTER SECTION */
                  do i = snum to 1 by -1                          /* for all sections found above */
                      mm = LookForSections( '"["*', 'FF', found.i.end +1 ) /* first section after */
                      if mm > 0 then do
                          call DeleteLine SectionsFound.1.Start, SectionsFound.1.End - SectionsFound.1.Start +1
                          say '---> delete section from line nr.'SectionsFound.1.Start 'to line nr.'SectionsFound.1.End
                      end
                  end
              end
              if C.Attr == 'BEF' then do                                        /* BEFORE SECTION */
                  do i = snum to 1 by -1                          /* for all sections found above */
                      mm = LookForSections( '"["*', 'LF', 1, found.i.start -1 )/* first section before */
                      if mm > 0 then call DeleteLine SectionsFound.1.Start, SectionsFound.1.End - SectionsFound.1.Start +1
                          say '---> delete section from line nr.'SectionsFound.1.Start 'to line nr.'SectionsFound.1.End
                  end
              end
          end
          otherwise NOP
      end
   end
   otherwise NOP
end
return 0
/**************************************************************************************************/
Save: procedure expose Line. C. SAVE. Color.
select
   when C.Type1 == 'LINE' then do
       m = LookForLines( C.What.1, C.Option1 )
       if m > 0 then do
           j = LinesFound.1
           SAVE.1 = Line.j
           SAVE.0 = 1
           say '---> save line nr.'j
       end
       else do
           say Color.yellow||'---> target line not found:'||Color.white C.What.1
       end
   end
   when C.Type1 == 'SECTION' then do
       snum = LookForSections( C.What.1, C.Option1, 1 )
       if snum > 0 then do
           j = 0
           do i = SectionsFound.1.Start to SectionsFound.1.End
               j = j+1
               SAVE.j = Line.i
           end
           SAVE.0 = j
           say '---> save section from linemr.'SectionsFound.1.Start 'to line nr.'SectionsFound.1.End
       end
       else do
           say Color.yellow||'---> target section not found:'||Color.white C.What.1
       end
   end
   otherwise NOP
end
return 0
/**************************************************************************************************/
LookForStrings: procedure expose C. Line. StringsFound. Color.
type   = arg(1)
ss     = arg(2)                                                         /* original search string */
option = arg(3)
lnum   = arg(4)                                                        /* number of line in Line. */

CaseSens  = 0
ssupper = translate(ss)
if type == 'ID' then do
    if pos('C',ssupper) == 1 then CaseSens  = 1
    if CaseSens  == 1 then ss = substr(ss,2)
end
/*------------------------------------------------------------------------------------------------*/
if type == 'ID' then do /* string has to be parsed for "" cause it is out of the C.Option1 string */
    i = 0                                                                  /* parse search string */
    do while length(ss) > 0
        ss = strip(ss)
        i = i+1
        if substr(ss,1,1) == '*' then do
            ss.i = '*'
            parse value ss with '*'ss
        end
        else do
            parse value ss with '"'ss.i'"' ss
            do while substr(ss,1,1) == '"'                   /* handle "" as " within a substring */
                parse value ss with '"'tmp'"' ss
                ss.i = ss.i || '"' || tmp
            end
        end
    end
    ss.0 = i                                                              /* search stem complete */
end
else do
    ss.0 = 1
    ss.1 = ss
end
if CaseSens == 0 then do                                           /* make it uppercase if needed */
    do i = 1 to ss.0
        ss.i = translate( ss.i )
    end
end                                                                  /* end parsing search string */

/*------------------------------------------------------------------------------------------------*/
if ss.0 > 1 | ss.0 == 0 then do
    say Color.red||'invalid search string (several lines not possible)'||Color.white
    return 0
end

ss = ss.1
if CaseSens  == 0 then ss = translate(ss)               /* make it uppercase if not 'C' specified */

tmpline = Line.lnum
if CaseSens == 0 then tmpline = translate( tmpline )
found = 0
sstart = 1
do while pos(ss,tmpline,sstart) > 0 & sstart <= length( tmpline )
    found = found+1
    StringsFound.found.Start = pos(ss,tmpline,sstart)
    StringsFound.found.End   = StringsFound.found.Start + length(ss) -1
    sstart = StringsFound.found.End + 1
end
StringsFound.0 = found
if StringsFound.0 > 0 then do
    if pos('LF',option) > 0 then do
        k = StringsFound.0
        StringsFound.1.Start = StringsFound.k.Start
        StringsFound.1.End   = StringsFound.k.End
        StringsFound.0 = 1
    end
    if pos('FF',option) > 0 then do
        StringsFound.0 = 1
    end
end

return StringsFound.0

/**************************************************************************************************/
DeleteLine: procedure expose Line.
parse arg number, count
    if number = 0 then return
    if number > Line.0 then return
    if number = Line.0 then do
        Line.0 = Line.0 -1
        return
    end
    do i = number to Line.0 - count
        j = i + count
        Line.i = Line.j
    end
    Line.0 = Line.0 - count
return
/**************************************************************************************************/
InsertLine: procedure expose Line. Insert.                        /* inserts always after "where" */
parse arg where
    if Insert.0 = 0 then return
    if where > Line.0 then return

    tindex = Line.0 + Insert.0
    sindex = Line.0
    do i = Line.0 to where by -1
        Line.tindex = Line.sindex
        tindex = tindex-1
        sindex = sindex-1
    end
    j=1
    do i = where+1 to where+Insert.0
        Line.i = Insert.j
        j = j+1
    end
    Line.0 = Line.0 + Insert.0
return
/**************************************************************************************************/
LookForSections: procedure expose Line. LinesFound. C. SectionsFound.
search = arg(1)                                                                /* search criteria */
option = arg(2)                                                                  /* search option */
sstart = arg(3)                                                       /* line to start the search */
send   = arg(4)
SectionsFound.0 = 0

if sstart == '' then sstart = 1
if send   == '' then send   = Line.0

special = ''
if translate( search ) == 'FIRST' then do
    search = '"["*'
    special = 'FIRST'
end
if translate( search ) == 'LAST' then do
    search = '"["*'
    option = 'EF'
    special = 'LAST'
end
if datatype( search ) == 'NUM' then do
    special = search
    search = '"["*'
    option = 'EF'
end

SectionID = '['                                        /* character indicating start of a section */

SectionsFound.0 = LookForLines( search, option, sstart, send )
if SectionsFound.0 == 0 then return 0                                         /* no section found */

do i = 1 to SectionsFound.0
    SectionsFound.i.Start = LinesFound.i                          /* start line number of section */
    do j = SectionsFound.i.Start + 1 to Line.0                     /* from section to end of file */
        if j == Line.0 then do                                    /* section name is on last line */
            SectionsFound.i.End = j
            iterate i
        end
        else if substr(strip(Line.j),1,1) == SectionID then do
            SectionsFound.i.End = j-1                         /* end of section is one line above */
            k = j-1
            do while Line.k == ''
                SectionsFound.i.End = k-1
                k = k-1
            end
            iterate i
        end
    end
end
if special = 'FIRST' & SectionsFound.0 > 0 then SectionsFound.0 = 1
if special = 'LAST'  & SectionsFound.0 > 0 then do
    j = SectionsFound.0
    SectionsFound.1.Start = SectionsFound.j.Start
    SectionsFound.1.End   = SectionsFound.j.End
    SectionsFound.0 = 1
end
if datatype( special ) == 'NUM' & SectionsFound.0 > 0 then do
    j = special
    SectionsFound.1.Start = SectionsFound.j.Start
    SectionsFound.1.End   = SectionsFound.j.End
    SectionsFound.0 = 1
end
return SectionsFound.0
/******************************************************************************/
/* LookForLines: procedure expose Line. LinesFound.                           */
/*                                                                            */
/* exposes     Line.      : lines of read file                                */
/*             LinesFound. : returns all found linenumbers                    */
/* arguments:  ss : search string, e.g. 'CS * "PATH" * "com\dLL" * "Dll;"'    */
/*             op : options (FF, LF, EF)                                      */
/*             sstart: search start line (optional)                           */
/*             send:   search end line   (optional)                           */
/* return      number of matches found                                        */
/*                                                                            */
/* example to call: numfound = LookForLines( 'CS* "PATH="', 'FF' )            */
/*                                                                            */
/******************************************************************************/
LookForLines: procedure expose Line. LinesFound.
ss     = arg(1)
op     = arg(2)
sstart = arg(3)
send   = arg(4)
if sstart == '' then sstart = 1
if send   == '' then send   = Line.0

LinesFound.0 = 0

if translate(ss) == 'FIRST' then do                              /* handle FIRST search parameter */
    if Line.0 > 0 then do                                         /* file exists and is not empty */
        LinesFound.0 = 1
        LinesFound.1 = 1
        return 1
    end
    else return 0                                                         /* file is empty or new */
end
if translate(ss) == 'LAST' then do                                /* handle LAST search parameter */
    LinesFound.0 = 1
    LinesFound.1 = Line.0
    return 1
end
if datatype(ss) == 'NUM' then do                              /* handle numeric search parameters */
    LinesFound.0 = 1
    LinesFound.1 = strip(ss)
    return 1
end

CaseSens  = 0
LineStrip = 0                                                    /* handle C and S string options */
ssupper = translate(ss)
if pos('C',ssupper) == 1 | (pos('C',ssupper) == 2 & pos('S',ssupper) == 1) then CaseSens  = 1
if pos('S',ssupper) == 1 | (pos('S',ssupper) == 2 & pos('C',ssupper) == 1) then LineStrip = 1
if CaseSens  == 1 then ss = substr(ss,2)
if LineStrip == 1 then ss = substr(ss,2)
/*------------------------------------------------------------------------------------------------*/
i = 0                                                                      /* parse search string */
do while length(ss) > 0
    ss = strip(ss)
    i = i+1
    if substr(ss,1,1) == '*' then do
        ss.i = '*'
        parse value ss with '*'ss
    end
    else do
        parse value ss with '"'ss.i'"' ss
        do while substr(ss,1,1) == '"'                       /* handle "" as " within a substring */
            parse value ss with '"'tmp'"' ss
            ss.i = ss.i || '"' || tmp
        end
    end
end
ss.0 = i                                                                  /* search stem complete */
if CaseSens == 0 then do                                           /* make it uppercase if needed */
    do i = 1 to ss.0
        ss.i = translate( ss.i )
    end
end                                                                  /* end parsing search string */
/*------------------------------------------------------------------------------------------------*/
k = 0
/* do i = 1 to Line.0 */                                                 /* for all lines of file */
do i = sstart to send                                                    /* for all lines of file */
    tmpline = Line.i
    if LineStrip == 1 then tmpline = strip( tmpline )                                    /* strip */
    if CaseSens  == 0 then tmpline = translate( tmpline )                         /* uppercase it */

    startpos = 1
    wildcard = 0
    do j = 1 to ss.0                                                     /* for all search tokens */
        if ss.j == '*' then wildcard = 1
        else do
            foundpos = pos(ss.j,tmpline,startpos)
            if foundpos == 0 then do
                iterate i                                                       /* take next line */
            end
            else do
                if wildcard == 1 then do
                    if foundpos < startpos then iterate i                       /* take next line */
                end
                else do
                    if foundpos <> startpos then iterate i                      /* take next line */
                end
                startpos = foundpos + length(ss.j)
            end
            wildcard = 0
        end
    end

    if wildcard == 0 then do                                    /* no "*" at end of search string */
        j = ss.0
        foundpos = lastpos(ss.j,tmpline)                            /* exact match at end of line */
        if foundpos + length(ss.j) -1  == length(tmpline) then matchingend = 1
                                                          else matchingend = 0
    end
    if (wildcard == 1) | (matchingend == 1) then do                               /* line matches */
        k = k+1
        LinesFound.k = i                                                   /* note the found line */
        if pos('FF',op) > 0 then leave                                  /* first found is enougth */
    end
end
LinesFound.0 = k                                                         /* number of found lines */

if pos('LF',op) > 0 & k > 0 then do                                            /* only last found */
    LinesFound.1 = LinesFound.k
    LinesFound.0 = 1
end
return LinesFound.0                                               /* return number of found lines */
/**************************************************************************************************/
ValidateOption: procedure expose C. Option.                       /* only syntax check of options */
C.Option1    = CheckOption(C.Option1)
if C.Option1 == -1 then return -1
C.Option2    = CheckOption(C.Option2)
if C.Option2 == -1 then return -1
C.Attr       = CheckOption(C.Attr)
if C.Attr    == -1 then return -1
return ''
/**************************************************************************************************/
CheckOption: procedure expose Option. Color.                      /* only syntax check of options */

popt = arg(1)                                          /* sets the short version names of options */

    startpos = pos('"', popt)                                                 /* look for first " */
    endpos   = lastpos('"',popt)                                               /* look for last " */
    if  startpos <> 0 then do
        endpos = lastpos( '"',popt)
        string = substr(popt,startpos,endpos-startpos+1)                    /* string between " " */
        if translate(substr(popt,startpos-1,1)) == 'C' then do              /* look for leading c */
            string = 'C' || string
            startpos = startpos-1
        end
        popt = substr(popt,1,startpos-1) '$ID$' substr(popt,endpos+1)     /* placeholder instead */
    end
    else string = ''                                                             /* no " " string */

    i = 0
    do while pos(',',popt) <> 0                                      /* divide opt string in stem */
        i = i+1
        parse value popt with opt.i ',' popt
        popt  = strip(popt)
    end
    if popt <> '' then do
        i = i+1
        opt.i = popt
    end
    opt.0 = i                                                                /* number of options */
    do i = 1 to opt.0                                                /* do for all option strings */
        tmpo = ''
        do k = 1 to words(opt.i)                                       /* for all option subwords */
            ooo = translate(word(opt.i,k))
            if strip(ooo) == '$ID$' then do
                tmpo = tmpo string
                found = 1
                leave
            end
            found = 0
            do j = 1 to Option.0                                      /* for all possible options */
                if wordpos(ooo,Option.j) <> 0 then do                                /* opt found */
                    tmpo = tmpo word(Option.j,1)
                    found = 1
                    leave
                end
            end
            if found == 0 then return -1                                      /* option not found */
        end
        opt.i = strip(tmpo)
    end
    tmp = ''
    do i = 1 to opt.0                                            /* prepare options return string */
        if tmp <> '' then tmp = tmp','
        tmp = tmp||opt.i
    end
return tmp                                                  /* return concatenates options string */
/**************************************************************************************************/
ValidateCommand: procedure expose C. Color.
/*        0        1         2         3         4         5         6         7         8         9        10        11        12        13 */
/*        1234567890123456789012345678901234567890123456789012345678901234567890123456789012345678901234567890123456789012345678901234567890 */
val.1  = ''
val.2  = ''
val.3  = 'DEL STRING   id    [FF|LF|EF]                                                  IN     LINE     id  [FF|LF|EF]'
val.4  = 'DEL LINE     id    [FF|LF|EF]                                                  -      -        -   -'
val.5  = 'DEL LINE      -    -                                                           BEF    LINE     id  [FF|LF|EF]'
val.6  = 'DEL LINE      -    -                                                           AFT    LINE     id  [FF|LF|EF]'
val.7  = 'DEL LINE     id    [FF|LF|EF]                                                  IN     SECTION  id  [FF|LF|EF]'
val.8  = 'DEL SECTION  id    [FF|LF|EF]                                                  -      -        -   -'
val.9  = 'DEL SECTION   -    -                                                           AFT    SECTION  id  [FF|LF|EF]'
val.10 = 'DEL SECTION   -    -                                                           BEF    SECTION  id  [FF|LF|EF]'

val.11 = 'ADD STRING    a    [ATB|ATE|AFT [FF|LF|EF] id|BEF [FF|LF|EF] id],[NOX|DOX]     IN     LINE     id  [FF|LF|EF]'
val.12 = 'ADD TOKEN     a    [ATB|ATE|AFT [FF|LF|EF] id|BEF [FF|LF|EF] id],[NOX|DOX]     IN     LINE     id  [FF|LF|EF]'
val.13 = 'ADD LINE      a    [NOX|NOXF|DOX],[ICR ATB|ICR ATE|NOF]                        AFT    LINE     id  [FF|LF|EF]'
val.14 = 'ADD LINE      a    [NOX|NOXF|DOX],[ICR ATB|ICR ATE|NOF]                        BEF    LINE     id  [FF|LF|EF]'
val.15 = 'ADD LINE      a    [ATB|ATE],[NOX|NOXS|DOX]                                    IN     SECTION  id  [FF|LF|EF],[ICR ATB|ICR ATE|NOF]'
val.16 = 'ADD SECTION   a    [NOX|NOXF|DOX],[ICR ATB|ICR ATE|NOF]                        AFT    LINE     id  [FF|LF|EF]'
val.17 = 'ADD SECTION   a    [NOX|NOXF|DOX],[ICR ATB|ICR ATE|NOF]                        BEF    LINE     id  [FF|LF|EF]'
val.18 = 'ADD SECTION   a    [NOX|NOXF|DOX],[ICR ATB|ICR ATE|NOF]                        AFT    SECTION  id  [FF|LF|EF],[NOF]'
val.19 = 'ADD SECTION   a    [NOX|NOXF|DOX],[ICR ATB|ICR ATE|NOF]                        BEF    SECTION  id  [FF|LF|EF],[NOF]'

val.20 = 'REP STRING   id    [FF|LF|EF]                                                  WIT    STRING   a   -'
val.21 = 'REP LINE     id    [FF|LF|EF],[ICR ATB|ICR ATE|NOF]                            WIT    LINE     a   -'
val.22 = 'REP SECTION  id    [FF|LF|EF],[ICR ATB|ICR ATE|NOF]                            WIT    SECTION  a   -'

val.23 = 'SAV LINE     id    [FF|LF]                                                     -      -        -   -'
val.24 = 'SAV SECTION  id    [FF|LF]                                                     -      -        -   -'

val.25 = 'OPEN         id    -                                                           -      -        -   -'
val.26 = 'CLOSE        id    -                                                           -      -        -   -'
val.27 = 'REX          a     -                                                           -      -        -   -'
val.0  = 27
/*        0        1         2         3         4         5         6         7         8         9        10        11        12        13 */
/*        1234567890123456789012345678901234567890123456789012345678901234567890123456789012345678901234567890123456789012345678901234567890 */
def.1  = ''
def.2  = ''
def.3  = '                   FF                                                                              FF'
def.4  = '                   FF                                                                              -'
def.5  = '                   -                                                                               FF'
def.6  = '                   -                                                                               FF'
def.7  = '                   FF                                                                              FF'
def.8  = '                   FF                                                                              -'
def.9  = '                   -                                                                               FF'
def.10 = '                   -                                                                               FF'

def.11 = '                   ATE,NOX                                                                         FF'
def.12 = '                   ATE,NOX                                                                         FF'
def.13 = '                   NOX,NOF                                                                         FF'
def.14 = '                   NOX,NOF                                                                         FF'
def.15 = '                   ATE,NOX                                                                         FF,ICR ATB'
def.16 = '                   NOX,NOF                                                                         FF'
def.17 = '                   NOX,NOF                                                                         FF'
def.18 = '                   NOX,NOF                                                                         FF,NOF'
def.19 = '                   NOX,NOF                                                                         FF,NOF'

def.20 = '                   FF'
def.21 = '                   FF,NOF                                                                          -'
def.22 = '                   FF,NOF                                                                          -'

def.23 = '                   FF                                                                              -'
def.24 = '                   FF                                                                              -'

def.25 = '                   -                                                                               -'
def.26 = '                   -                                                                               -'
def.27 = '                   -                                                                               -'

def.0  = 27


/*------------------------------------------------------------------------------------------------*/
if C.Cmd == 'OPEN' | C.Cmd == 'CLOSE' then do
    if C.What.0 = 1 then return 0
                    else return 'no file name specified'
end
if C.Cmd == 'REX' then do
    if C.What.0 > 0 then return 0
                    else return 'no REXX code specified'
end


res.0 = 0
j     = 0
do i = 1 to val.0                                                       /* look for Cmd and Type1 */
    if C.Cmd == word(val.i,1) & C.Type1 == word(val.i,2) then do
        j = j+1
        res.j = i
    end
end
res.0 = j
if j == 0 then do
    return 'invalid type:' C.Type1
end

ok = 0
do i = 1 to res.0
    j = res.i
    if (C.Attr  == word(substr(val.j,80),1) &,                            /* Attr and Type2 exist */
        C.Type2 == word(substr(val.j,87),1)) |,
       ((C.Attr  == '' & word(substr(val.j,80),1) == '-') &,
       (C.Type2 == '' & word(substr(val.j,87),1) == '-'))
    then do
        ok = 1
        foundval = val.j                                    /* the only matching command template */
        founddef = def.j
    end
end
if ok == 0 then do
    return 'invalid attribute or type:' C.Attr C.Type2
end

if word(foundval,3) == '-' & C.What.0 <> 0 then return 'invalid id string 1:' C.What.1
if substr(foundval,96,1) == '-' & C.Search <> '' then return 'invalid id string 2:' C.Search

C.Option1 = AnalyseOptionAndSetDefault( C.Option1, substr(foundval,20,60),  strip(substr(founddef,20,60)))
if C.Option1 == -1 then return 'invalid option 1'
C.Option2 = AnalyseOptionAndSetDefault( C.Option2, substr(foundval,100,60), strip(substr(founddef,100,60)))
if C.Option2 == -1 then return 'invalid option 2'

/*** show parsed line ***/
ssss = ''; do i=1 to C.What.0; if ssss=='' then ssss=C.What.i;else ssss=ssss||',' C.What.i; end
/* if C.Cmd == 'REP'
then say 'Parsed  :' C.Cmd C.Type1 C.Search       '('||C.Option1||')' C.Attr C.Type2 '"'||ssss||'"' '('||C.Option2||')'
else say 'Parsed  :' C.Cmd C.Type1 '"'||ssss||'"' '('||C.Option1||')' C.Attr C.Type2 C.Search '('||C.Option2||')' */


/*** say '-----------------------'                  ***/
/*** say 'C.Cmd     ='  '|'||C.Cmd||'|'             ***/
/*** say 'C.Type1   ='  '|'||C.Type1||'|'           ***/
/*** say 'C.What.0  ='  '|'||C.What.0||'|'          ***/
/*** do i=1 to C.What.0                             ***/
/***     say 'C.What.'i'  ='  '|'||C.What.i||'|'    ***/
/*** end                                            ***/
/*** say 'C.Option1 ='  '|'||C.Option1||'|'         ***/
/*** say 'C.Attr    ='  '|'||C.Attr||'|'            ***/
/*** say 'C.Type2   ='  '|'||C.Type2||'|'           ***/
/*** say 'C.Search  ='  '|'||C.Search||'|'          ***/
/*** say 'C.Option2 ='  '|'||C.Option2||'|'         ***/
/*** say '-----------------------'                  ***/
return 0
/**************************************************************************************************/
AnalyseOptionAndSetDefault: procedure                                    /* validates the options */
orgopt = arg(1)                                     /* meens, if option is allowed in the context */
refopt = arg(2)
defopt = arg(3)

opt = orgopt
if opt == '' & RefOpt <> '-' then return defopt                /* return default if no option set */

startpos = pos('"', opt)                                                      /* look for first " */
endpos   = lastpos('"',opt)                                                    /* look for last " */
if  startpos <> 0 then do
    endpos = lastpos( '"',opt)
    string = substr(opt,startpos,endpos-startpos+1)                         /* string between " " */
    if translate(substr(opt,startpos-1,1)) == 'C' then do                   /* look for leading c */
        string = 'C' || string
        startpos = startpos-1
    end
    opt = substr(opt,1,startpos-1) 'id' substr(opt,endpos+1)               /* placeholder instead */
end
else string = ''                                                                 /* no " " string */

/* put referen and default options into stem */
i = 0; do while refopt <> ''; i = i+1; parse value refopt with refopt.i','refopt; end; refopt.0 = i
i = 0; do while defopt <> ''; i = i+1; parse value defopt with defopt.i','defopt; end; defopt.0 = i
i = 0; do while opt    <> ''; i = i+1; parse value opt    with opt.i','opt;       end; opt.0    = i

do i = 1 to opt.0
    found = 0
    do j = 1 to refopt.0
        if OptionStringMatch( opt.i, refopt.j ) <> 0 then do             /* possible option found */
            found = 1
            defopt.j = ''                                   /* exclude the related default option */
            iterate i
        end
    end
    if found == 0 then do
        return -1
    end
end
do i = 1 to defopt.0                                         /* if there are default options left */
    if defopt.i <> '' & defopt.i <> '-' then orgopt = orgopt','defopt.i
end


return orgopt
/**************************************************************************************************/
OptionStringMatch: procedure
str1 = arg(1)                                                             /* option to be checked */
str2 = arg(2)                                                                 /* reference option */
do i = 1 to words(str1)                                   /* checks for right sequence of options */
    pp = pos(word(str1,i),str2)
    if pp == 0 then return 0
    else do
        str2 = substr(str2,pp+length(word(str1,i)))
    end
end
return pp
/**************************************************************************************************/
HandleParametersAndInputErrors: procedure
    args = strip(arg(1))
    if args == '' then do
        call Usage
        exit -1
    end
    if words(args) == 1 then do                                               /* filename assumed */
        filename = word(args,1)
        if stream(filename,'c','query exists') == '' then do
            call usage
            exit -1
        end
        else return filename
    end
    else do
        queue 'OPEN' word(args,1)
        queue subword(args,2)
        queue 'CLOSE' word(args,1)
    end
return ''
/**************************************************************************************************/
Usage: procedure
say 'Illegal call to MODIFY'
say
say 'Usage:  Modify <commandfile>                  all commands in the file'
say '        Modify <filename>    <command>        command line version'

return 0
/**************************************************************************************************/
Logo: procedure
/* parse source . . srcname
call SysFileTree srcname, 'file', 'FT'
parse value(file.1) with yy'/'mm'/'dd'/'hh'/'ii .
sfile.1 = subword(sfile.1,1,2)
lastchange = 'This program was last changed on' dd'.'mm'.19'yy hh':'ii
title = 'MODIFY Ver. 1.1                                    S.Suter IBM, 04.08.1996'
say ''copies('',length(title)+2)''
say '' title ''
say '' right(lastchange,length(title)) ''
say ''copies('',length(title)+2)'' */
return 0
/**************************************************************************************************/
