/* Parts Copyright (C)2001 by Holger Veit
 * Parts Copyright (C)2001 by Michael Zolk
 */

#include <alloca.h>
#include <string.h>
#define INCL_REXXSAA
#define INCL_DOSPROCESS
#define INCL_DOSFILEMGR
#include <os2.h>
#include <sys/param.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <string.h>
#include <stdlib.h>
#include <io.h>
#include <fcntl.h>
#include <sys/ead.h>
#include <sys/ea.h>

static PSZ UX2Version = "0.1";

static PSZ RxFuncTable[] = {
  "UX2GetVersion",
  "UX2UnixRoot",
  "UX2MkSymLink",
  "UX2Canonicalize",
  "UX2UnixName",
  "UX2DelSymLink",
  "UX2ReadSymLink",
  "UX2Chmod",
  "UX2Chown",
  "UX2Chgrp",
  "UX2Mknod",
  "UX2DevMajor",
  "UX2DevMinor",
  "UX2DirName",
  "UX2BaseName",
  "UX2IsDir",
  "UX2IsFile",
  "UX2IsCharDev",
  "UX2IsBlkDev",
  "UX2IsSymLink",
  "UX2Mount",
  "UX2Umount",
  "UX2DirExists",
  "UX2FileExists",
  "UX2MkDir",
  "UX2DelDir",
  "UX2CreateDir",
  "UX2Uid",
  "UX2Gid",
  "UX2Gid",
  "UX2Pid",
  "UX2PPid",
  "UX2Umask"
};

/* The virtual toplevel of the UnixOS/2 system */
char *unixroot = NULL;
int unixrootlen;

/* temporary hacks, will go away:
 * in the future, some SES system will provide a UID/GID/umask for
 * each process. For now, you can individually set owner/group/umask
 * with the appropriate UX2 functions. In the future, this will no
 * longer work if you don't have the appropriate privilege
 */
long uid = 0;
long gid = 0;
long umaskval = 0;

/* provide PID/PPID to allow building temporary names */
long pid = -1;
long ppid = -1;

/**
 * This is the basic Inode structure.
 * it is here written like a C++ class in C (to avoid C++ overhead)
 */
typedef struct _inode {
  long size;
  long acl;
  long uid;
  long gid;
  long major;
  long minor;
  long attrib;
  long crlf_mode;
  int symlink_off;
  char data[1];
} UX2Inode;

#define INODE_FILE FILE_NORMAL
#define INODE_DIR  FILE_DIRECTORY
#define INODE_BDEV 0x10000
#define INODE_CDEV 0x20000
#define INODE_LINK 0x40000
#define symlink(in) (in->data+in->symlink_off)

UX2Inode* inode_new(int size) {
  long len = sizeof(UX2Inode)+size;
  UX2Inode* i = (UX2Inode*)malloc(len);
  i->size = len;
  i->acl = 0777 & ~umaskval;
  i->uid = uid; /* hack */
  i->gid = gid; /* hack */
  i->major = 0;
  i->minor = 0;
  i->attrib = INODE_FILE;
  i->symlink_off = 0;
  i->data[0] = 0;
}

void inode_delete(UX2Inode* this) {
  free(this);
}

UX2Inode* inode_read(char* filename) {
  _ead ead;
  FILESTATUS3 fs3;
  UX2Inode* this = NULL;
  int newi = 0;

  APIRET rc = DosQueryPathInfo(filename,FIL_STANDARD,&fs3,sizeof(FILESTATUS3));
  if (rc == 0) {
    /* file seems to exist */
    ead = _ead_create();
    if ((rc=_ead_read(ead,filename,0,0)) == 0) {
      /* read EAs successful */
      int idx = _ead_find(ead,".UX2INODE");
      if (idx >= 0) {
        /* copy the inode */
        int sz = _ead_value_size(ead,idx);
        this = inode_new(sz);
        memcpy(this,_ead_get_value(ead,idx),sz);
      } else {
        /* create a new one */
        this = inode_new(0);
        this->attrib = fs3.attrFile;
        newi = 1;
      }
    } else {
      /* create a new one */
      this = inode_new(0);
      this->attrib = fs3.attrFile;
      newi = 1;
    }
    _ead_destroy(ead);
   if (newi)
     inode_write(this,filename);
   return this;
  }
  return NULL;
}

/* requires "this" to be !NULL */
int inode_write(UX2Inode* this, char* filename) {
  APIRET rc;
  struct _ea ea;
  int sz = this->size;

  ea.flags = 0x80;
  ea.size = sz;
  ea.value = malloc(sz);
  memcpy(ea.value,this,sz);
  rc = _ea_put(&ea,filename,0,".ux2inode");
  free(ea.value);
  return rc;
}

static char* _translate(char* s) {
  char* r;
  for (r=s; *r; r++)
    if (*r == '\\') *r = '/';
  return s;
}

/* make a full pathname without any ".." in it */
static char* _canon(char* f) {

  static char path[260];
  APIRET rc = DosQueryPathInfo(f,FIL_QUERYFULLNAME,path,sizeof(path));
  if (rc == 0)
    return _translate(path);
  return "";
}

/**
 * this is the DLL init/terminate routine.
 */
unsigned long _DLL_InitTerm(ULONG modhndl, ULONG flag) {
  switch (flag) {
  case 0:
    if (_CRT_init() != 0)
      return 0;
    __ctordtorInit();
    unixroot = getenv("UNIXROOT");
    if (unixroot == NULL) {
      ULONG cbw;
      char* emsg = "Error from UX2UTIL.DLL: UNIXROOT variable is not set";
      DosWrite(2,emsg,sizeof(emsg),&cbw);
      return 0;
    }
    _translate(unixroot);
    unixrootlen = strlen(unixroot);
    if (unixroot[unixrootlen-1] == '/')
      unixroot[unixrootlen-1] = 0;

    return 1;
  case 1:
    __ctordtorTerm();
    _CRT_term();
    return 1;
  default:
    return 0;
  }
}

/* call UX2LoadFuncs() */
ULONG UX2LoadFuncs(PSZ name,LONG argc,RXSTRING* argv,
                   PSZ queuename, RXSTRING* ret) {

  int i;

  ret->strlength = 0;
  if (argc > 0)
    return RXFUNC_MODNOTFND;

  for (i=0; i< (sizeof(RxFuncTable)/sizeof(PSZ)); i++)
    RexxRegisterFunctionDll(RxFuncTable[i],
                            "UX2UTIL",
                            RxFuncTable[i]);
  return RXFUNC_OK;
}

/* version = UX2GetVersion() */
ULONG UX2GetVersion(PSZ name,LONG argc,RXSTRING* argv,
                    PSZ queue,RXSTRING* ret) {

  ret->strlength = 0;
  if (argc > 0)
    return RXFUNC_MODNOTFND;
  strcpy(ret->strptr,UX2Version);
  ret->strlength = strlen(UX2Version);
  return RXFUNC_OK;
}

static int _set_str(RXSTRING* ret,char* str) {
  strcpy(ret->strptr,str);
  ret->strlength = strlen(str);
  return RXFUNC_OK;
}

static int _set_rc(RXSTRING* ret,int rc) {
  sprintf(ret->strptr,"%d",rc);
  ret->strlength = strlen(ret->strptr);
  return RXFUNC_OK;
}

static int _set_rco(RXSTRING* ret,int rc) {
  sprintf(ret->strptr,"0%o",rc);
  ret->strlength = strlen(ret->strptr);
  return RXFUNC_OK;
}

/* rc = UX2DelSymLink(file) */
ULONG UX2DelSymLink(PSZ name,LONG argc,RXSTRING* argv,
                    PSZ queue,RXSTRING* ret) {
  APIRET rc;

  ret->strlength = 0;
  if (argc != 1)
    return RXFUNC_MODNOTFND;

  return _set_rc(ret,DosDelete(RXSTRPTR(argv[0])));
}

/* unixroot = UX2UnixRoot() */
ULONG UX2UnixRoot(PSZ name,LONG argc,RXSTRING* argv,
                  PSZ queue,RXSTRING* ret) {
  ret->strlength = 0;
  if (argc != 0)
    return RXFUNC_MODNOTFND;
  return _set_str(ret,unixroot);
}

/* strip off a UNIXROOT prefix from path */
/* uxpath = UX2UnixName("c:/unixroot/some/path") */
/* XXX this function might in future also respect mount points,
 * e.g. a:/file.xxx -> /floppy/file.xxx, if a: is mounted on /floppy
 */
ULONG UX2UnixName(PSZ name,LONG argc,RXSTRING* argv,
                  PSZ queue,RXSTRING* ret) {
  char *f;

  ret->strlength = 0;
  if (argc != 1)
    return RXFUNC_MODNOTFND;

  f = RXSTRPTR(argv[0]);
  if (*f == '/' || *f == '\\') {
    _translate(f);
    return _set_str(ret,f);
  }

  f = _canon(f);
  if (strlen(f) >= unixrootlen &&
      strnicmp(f,unixroot,unixrootlen) == 0 &&
      f[unixrootlen]=='/')
    return _set_str(ret,f+unixrootlen);
  else
    return _set_str(ret,f);
}

/* rc = UX2MkSymLink(nameoflink,pointto [,delfilefirst]) */
ULONG UX2MkSymLink(PSZ name,LONG argc,RXSTRING* argv,
                   PSZ queue,RXSTRING* ret) {

  APIRET rc = RXFUNC_OK;
  ULONG openflags = OPEN_ACTION_CREATE_IF_NEW | OPEN_ACTION_FAIL_IF_EXISTS;

  ret->strlength = 0;

  if (argc==3) {
    if (atol(RXSTRPTR(argv[2]))==1)
      openflags = OPEN_ACTION_CREATE_IF_NEW | OPEN_ACTION_REPLACE_IF_EXISTS;
    argc--;
  }
  if (argc==2) {
    char* linkname = RXSTRPTR(argv[0]);
    char* pointto = RXSTRPTR(argv[1]);

    HFILE hf;
    ULONG action,nbuf;
    char buf[300];
    UX2Inode *inode = inode_new(strlen(pointto));

    rc = DosOpen(linkname,&hf,&action,0,FILE_NORMAL,
                 openflags,
                 OPEN_ACCESS_WRITEONLY|OPEN_SHARE_DENYREADWRITE,NULL);
    if (rc==0) {
      sprintf(buf,"This file is a UnixOS/2 symlink.\r\n");
      if ((rc = DosWrite(hf,buf,strlen(buf),&nbuf))==0) {
        sprintf(buf,"Its relative target is <%s>.\r\n",pointto);
        if ((rc = DosWrite(hf,buf,strlen(buf),&nbuf))==0) {
          char *cname = _canon(pointto);
          sprintf(buf,"Its absolute target is <%s>.\r\n",cname);
          rc = DosWrite(hf,buf,strlen(buf),&nbuf);
        }
      }
      DosClose(hf);
      inode->attrib = INODE_LINK;
      strcpy(symlink(inode),pointto);
      if (rc != 0)
        DosDelete(linkname);
      else
        rc = inode_write(inode,linkname);
    }
  }
  return _set_rc(ret,rc);
}

/* path = UXCanonicalize(path) */
ULONG UX2Canonicalize(PSZ name,LONG argc,RXSTRING* argv,
                      PSZ queue,RXSTRING* ret) {
  char *s;

  ret->strlength = 0;
  if (argc != 1)
    return RXFUNC_MODNOTFND;

  s = RXSTRPTR(argv[0]);
  if (*s == '/' || *s == '\\') {
    char buf[260];
    sprintf(buf,"%s%s",unixroot,s);
    _translate(buf);
    return _set_str(ret,buf);
  } else
    return _set_str(ret,_canon(s));
}

/* link = UX2ReadSymLink(path) */
ULONG UX2ReadSymLink(PSZ name,LONG argc,RXSTRING* argv,
                     PSZ queue,RXSTRING* ret) {
  UX2Inode *inode;

  ret->strlength = 0;
  if (argc != 1)
    return RXFUNC_MODNOTFND;

  if ((inode=inode_read(RXSTRPTR(argv[0]))) != NULL) {
    _set_str(ret,symlink(inode));
    inode_delete(inode);
  }
  return RXFUNC_OK;
}

/* rc = UX2Chmod(filename,mode) */
/* octal_mode = UX2Chmod(filename) */
ULONG UX2Chmod(PSZ name,LONG argc,RXSTRING* argv,
               PSZ queue,RXSTRING* ret) {

  char *filename;
  UX2Inode* inode;
  APIRET rc = 0;

  ret->strlength = 0;
  if (argc == 0) return RXFUNC_MODNOTFND;

  filename = RXSTRPTR(argv[0]);
  if ((inode=inode_read(filename)) != NULL) {
    switch (argc) {
    case 1:
      _set_rco(ret,inode->acl);
      inode_delete(inode);
      return RXFUNC_OK;
    case 2:
      inode->acl = strtol(RXSTRPTR(argv[1]),NULL,0) & 07777;
      rc = inode_write(inode,filename);
      break;
    default:
      rc = RXFUNC_MODNOTFND;
    }
  } else
    rc = -1;
  inode_delete(inode);
  return _set_rc(ret,rc);
}

/* rc = UX2Chown(filename,uid) */
/* uid = UX2Chown(filename) */
ULONG UX2Chown(PSZ name,LONG argc,RXSTRING* argv,
               PSZ queue,RXSTRING* ret) {
  UX2Inode* inode;
  char* filename;
  APIRET rc = RXFUNC_OK;

  ret->strlength = 0;
  if (argc == 0) return RXFUNC_MODNOTFND;

  filename = RXSTRPTR(argv[0]);
  if ((inode=inode_read(filename)) != NULL) {
    switch (argc) {
    case 1:
      rc = inode->uid;
      break;
    case 2: /* set */
      inode->uid = strtol(RXSTRPTR(argv[1]),NULL,0);
      rc = inode_write(inode,filename);
      break;
    default:
      rc = RXFUNC_MODNOTFND;
    }
  }

  inode_delete(inode);
  return _set_rc(ret,rc);
}

/* rc = UX2Chgrp(filename,gid) */
/* gid = UX2Chgrp(filename) */
ULONG UX2Chgrp(PSZ name,LONG argc,RXSTRING* argv,
               PSZ queue,RXSTRING* ret) {
  UX2Inode* inode;
  char* filename;
  APIRET rc = RXFUNC_OK;

  ret->strlength = 0;
  if (argc == 0) return RXFUNC_MODNOTFND;

  filename = RXSTRPTR(argv[0]);
  if ((inode=inode_read(filename)) != 0) {
    switch (argc) {
    case 1:
      rc = inode->gid;
      break;
    case 2: /* set */
      inode->gid = strtol(RXSTRPTR(argv[1]),NULL,0);
      rc = inode_write(inode,filename);
      break;
    default:
      rc = RXFUNC_MODNOTFND;
    }
  }

  inode_delete(inode);
  return _set_rc(ret,rc);
}

/* rc = UX2Mknod(filename,{c|b},major,minor) */
ULONG UX2Mknod(PSZ name,LONG argc,RXSTRING* argv,
               PSZ queue,RXSTRING* ret) {

  UX2Inode *inode;
  char *filename;
  APIRET rc = RXFUNC_OK;

  ret->strlength = 0;
  if (argc != 4) return RXFUNC_MODNOTFND;

  filename = RXSTRPTR(argv[0]);
  inode = inode_new(0);
  switch (RXSTRPTR(argv[1])[0]) {
  case 'C':
  case 'c':
    inode->attrib = INODE_CDEV;
    break;
  case 'B':
  case 'b':
    inode->attrib = INODE_BDEV;
    break;
  default:
    rc = RXFUNC_MODNOTFND;
  }

  inode->major = strtol(RXSTRPTR(argv[2]),NULL,0);
  inode->minor = strtol(RXSTRPTR(argv[3]),NULL,0);
  if (inode->major < 0 || inode->minor < 0)
    rc = RXFUNC_MODNOTFND;

  if (rc == RXFUNC_OK) {
    HFILE hf;
    ULONG action;
    rc = DosOpen(filename,&hf,&action,0,FILE_NORMAL,
                 OPEN_ACTION_CREATE_IF_NEW | OPEN_ACTION_FAIL_IF_EXISTS,
                 OPEN_ACCESS_WRITEONLY|OPEN_SHARE_DENYREADWRITE,NULL);
    if (rc==0) {
      char buf[128];
      ULONG nbuf;
      sprintf(buf,"This file is a UnixOS/2 %s device.\r\n",
              (inode->attrib & INODE_BDEV) != 0 ? "BLOCK" : "CHAR");
      if ((rc = DosWrite(hf,buf,strlen(buf),&nbuf))==0) {
        sprintf(buf,"Its major is <%d>, its minor is <%d>\r\n",
                inode->major,inode->minor);
        rc = DosWrite(hf,buf,strlen(buf),&nbuf);
      }
      DosClose(hf);
      if (rc != 0)
        DosDelete(filename);
      else
        rc = inode_write(inode,filename);
    }
  }

  inode_delete(inode);
  return _set_rc(ret,rc);
}

/* major = UX2DevMajor(filename) */
ULONG UX2DevMajor(PSZ name,LONG argc,RXSTRING* argv,
                     PSZ queue,RXSTRING* ret) {
  UX2Inode *inode;
  APIRET rc = RXFUNC_OK;

  ret->strlength = 0;
  if (argc != 1) return RXFUNC_MODNOTFND;

  inode = inode_read(RXSTRPTR(argv[0]));
  rc = inode != 0 ? inode->major : 0;
  inode_delete(inode);
  return _set_rc(ret,rc);
}

/* minor = UX2DevMinor(filename) */
ULONG UX2DevMinor(PSZ name,LONG argc,RXSTRING* argv,
                  PSZ queue,RXSTRING* ret) {
  UX2Inode *inode;
  APIRET rc = RXFUNC_OK;

  ret->strlength = 0;
  if (argc != 1) return RXFUNC_MODNOTFND;

  inode = inode_read(RXSTRPTR(argv[0]));
  rc = inode != 0 ? inode->minor : 0;
  inode_delete(inode);
  return _set_rc(ret,rc);
}

/* dirname = UX2DirName(pathname) */
ULONG UX2DirName(PSZ name,LONG argc,RXSTRING* argv,
                 PSZ queue,RXSTRING* ret) {
  char *r,*s;

  ret->strlength = 0;
  if (argc != 1) return RXFUNC_MODNOTFND;

  r = _translate(RXSTRPTR(argv[0]));
  s = strrchr(r,'/');
  if (s == NULL)
      return _set_str(ret,".");
  *s = 0;
  return _set_str(ret,r);
}

/* basename = UX2BaseName(pathname [,ext]) */
ULONG UX2BaseName(PSZ name,LONG argc,RXSTRING* argv,
                  PSZ queue,RXSTRING* ret) {
  char *r,*s;

  ret->strlength = 0;
  if (argc == 1 || argc == 2) {
    r = _translate(RXSTRPTR(argv[0]));
    s = strrchr(r,'/');
    if (s==NULL) s = r;
    else s++;
    if (argc==2) {
      r = strstr(s,RXSTRPTR(argv[1]));
      if (r != NULL) *r = 0;
    }
    return _set_str(ret,s);
  } else
      return RXFUNC_MODNOTFND;
}

int _test_attrib(char* path, int flg) {
  int rc;
  UX2Inode* inode = inode_read(path);
  if (inode==NULL)
    rc = -1;
  else
    rc = (inode->attrib & flg) != 0 ? 1 : 0;
  inode_delete(inode);
  return rc;
}

/* rc = UX2IsDir(pathname)
 * 1=yes, 0=no, -1=not exist */
ULONG UX2IsDir(PSZ name,LONG argc,RXSTRING* argv,
               PSZ queue,RXSTRING* ret) {
  ret->strlength = 0;
  if (argc != 1)
    return RXFUNC_MODNOTFND;

  return _set_rc(ret,_test_attrib(RXSTRPTR(argv[0]),INODE_DIR));
}

ULONG UX2IsFile(PSZ name,LONG argc,RXSTRING* argv,
                PSZ queue,RXSTRING* ret) {
  APIRET rc;
  ret->strlength = 0;
  if (argc != 1)
    return RXFUNC_MODNOTFND;

  rc = _test_attrib(RXSTRPTR(argv[0]),
                    INODE_DIR|INODE_LINK|INODE_BDEV|INODE_CDEV);
  if (rc >= 0)
    rc = 1 - rc;
  return _set_rc(ret,rc);
}

ULONG UX2IsCharDev(PSZ name,LONG argc,RXSTRING* argv,
                   PSZ queue,RXSTRING* ret) {
  ret->strlength = 0;
  if (argc != 1)
    return RXFUNC_MODNOTFND;

  return _set_rc(ret,_test_attrib(RXSTRPTR(argv[0]),INODE_CDEV));
}

ULONG UX2IsBlkDev(PSZ name,LONG argc,RXSTRING* argv,
                    PSZ queue,RXSTRING* ret) {
  ret->strlength = 0;
  if (argc != 1)
    return RXFUNC_MODNOTFND;

  return _set_rc(ret,_test_attrib(RXSTRPTR(argv[0]),INODE_BDEV));
}

ULONG UX2IsSymLink(PSZ name,LONG argc,RXSTRING* argv,
                   PSZ queue,RXSTRING* ret) {
  ret->strlength = 0;
  if (argc != 1)
    return RXFUNC_MODNOTFND;

  return _set_rc(ret,_test_attrib(RXSTRPTR(argv[0]),INODE_LINK));
}

ULONG UX2Mount(PSZ name,LONG argc,RXSTRING* argv,
               PSZ queue,RXSTRING* ret) {
  return RXFUNC_MODNOTFND;
}

ULONG UX2Umount(PSZ name,LONG argc,RXSTRING* argv,
                PSZ queue,RXSTRING* ret) {
  return RXFUNC_MODNOTFND;
}

ULONG UX2DelDir(PSZ name,LONG argc,RXSTRING* argv,
                PSZ queue,RXSTRING* ret) {
  return RXFUNC_MODNOTFND;
}

static int _findpid() {
  if (pid == -1) {
    PPIB ppib = NULL;
    APIRET rc = DosGetInfoBlocks(NULL,&ppib);
    if (rc != 0)
      return 0;
    pid = ppib->pib_ulpid;
    ppid= ppib->pib_ulppid;
  }
  return 1;
}

/* pid = UX2Pid() */
ULONG UX2Pid(PSZ name,LONG argc,RXSTRING* argv,
                PSZ queue,RXSTRING* ret) {

  ret->strlength = 0;
  if (argc != 0)
    return RXFUNC_MODNOTFND;

  if (_findpid())
    _set_rc(ret,pid);
  return RXFUNC_OK;
}

/* pid = UX2PPid() */
ULONG UX2PPid(PSZ name,LONG argc,RXSTRING* argv,
                PSZ queue,RXSTRING* ret) {

  ret->strlength = 0;
  if (argc != 0)
    return RXFUNC_MODNOTFND;

  if (_findpid())
    _set_rc(ret,ppid);
  return RXFUNC_OK;
}

/***************************************************************
 * Note the following routines are temporary hacks. In the
 * future, there should be a system wide process management
 * that handles the real user and group identities, so these
 * routines might no longer return arbitrarily modifiable
 * local variables, and may even fail to change identities
 * due to insufficient privileges.
 **************************************************************/

/* rc = UX2Uid(newuid) */
/* uid = UX2Uid() */
ULONG UX2Uid(PSZ name,LONG argc,RXSTRING* argv,
             PSZ queue,RXSTRING* ret) {

  long u;
  ret->strlength = 0;
  switch(argc) {
  default:
    return RXFUNC_MODNOTFND;
  case 1:
    u = strtol(RXSTRPTR(argv[0]),NULL,0);
    uid = u;
    return _set_rc(ret,0);
  case 0:
    return _set_rc(ret,uid);
  }
}

/* rc = UX2Gid(newgid) */
/* gid = UX2Gid() */
ULONG UX2Gid(PSZ name,LONG argc,RXSTRING* argv,
             PSZ queue,RXSTRING* ret) {

  long g;
  ret->strlength = 0;
  switch(argc) {
  default:
    return RXFUNC_MODNOTFND;
  case 1:
    g = strtol(RXSTRPTR(argv[0]),NULL,0);
    gid = g;
    return _set_rc(ret,0);
  case 0:
    return _set_rc(ret,gid);
  }
}

/* rc = UX2UMask(newumask) */
/* octalumask = UX2UMask() */
ULONG UX2UMask(PSZ name,LONG argc,RXSTRING* argv,
             PSZ queue,RXSTRING* ret) {

  long u;
  ret->strlength = 0;
  switch(argc) {
  default:
    return RXFUNC_MODNOTFND;
  case 1:
    u = strtol(RXSTRPTR(argv[0]),NULL,0);
    umaskval = u;
    return _set_rc(ret,0);
  case 0:
    return _set_rco(ret,umaskval);
  }
}

/* check if a directory exists */
ULONG UX2DirExists(CHAR *name,
                  ULONG numargs,
                  RXSTRING args[],
                  CHAR *queuename,
                  RXSTRING *retstr)
{
  APIRET rc;

  retstr->strlength = 0;
  if (numargs != 1)
    return RXFUNC_MODNOTFND;

  rc = _test_attrib(RXSTRPTR(args[0]),INODE_DIR);
  if (rc == -1) rc = 0;
  return _set_rc(retstr,rc);
}

/* create a directory tree */
ULONG UX2CreateDir(CHAR *name,
                   ULONG argc,
                   RXSTRING argv[],
                   CHAR *queuename,
                   RXSTRING *ret)

{
    char *path,*r,*s;
    APIRET rc;

    if (argc != 1)
        return RXFUNC_MODNOTFND;

    /* get the real tree (canonicalize) */
    path = _canon(RXSTRPTR(argv[0]));

    r = path;
    do {
        s = strchr(r,'/');
        if (s) {
            *s = 0;
            /* we don't check the rc until the last component: if the last
             * create succeeds, then the tree should be there as well
             */
            rc = DosCreateDir(path,NULL);
            *s = '/';
            r = s+1;
        }
    } while (s);

    return _set_rc(ret,rc);
}

/* check if a file exists */
ULONG UX2FileExists(CHAR *name,
                   ULONG numargs,
                   RXSTRING args[],
                   CHAR *queuename,
                   RXSTRING *retstr)

{
  APIRET rc;

  retstr->strlength = 0;
  if (numargs != 1)
    return RXFUNC_MODNOTFND;

  rc = _test_attrib(RXSTRPTR(args[0]),
                    INODE_DIR|INODE_LINK|INODE_CDEV|INODE_BDEV);
  rc = rc==0 ? 1 : 0;
  return _set_rc(retstr,rc);
}

/**
 * Local variables:
 * tab-width: 8
 * c-basic-offset: 4
 * font-lock-mode: t
 * End:
 */

